"""Access wordslab-notebooks version, platform, urls, ports, directories (install / apps / data / models), and pre-installed models."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['WordslabNotebooks']

# %% ../nbs/00_core.ipynb 3
import os

# %% ../nbs/00_core.ipynb 5
class WordslabNotebooks:
    def __init__(self):
        # wordslab-notebooks version and platform
        self.version = os.environ["WORDSLAB_VERSION"]
        self.platform = os.environ["WORDSLAB_PLATFORM"]

        # wordslab-notebooks external urls
        self.url_dashboard = os.environ["DASHBOARD_URL"]
        self.url_openwebui = os.environ["OPENWEBUI_URL"]
        self.url_jupyterlab = os.environ["JUPYTERLAB_URL"]
        self.url_vscode = os.environ["VSCODE_URL"]
        self.url_userapp1 = os.environ["USER_APP1_URL"]
        self.url_userapp2 = os.environ["USER_APP2_URL"]
        self.url_userapp3 = os.environ["USER_APP3_URL"]
        self.url_userapp4 = os.environ["USER_APP4_URL"]
        self.url_userapp5 = os.environ["USER_APP5_URL"]

        # wordslab-notebooks internal ports
        self.port_ollama = 11434
        self.port_vllm = 8000
        self.port_docling = 5001   
        self.port_userapp1 = os.environ["USER_APP1_PORT"]
        self.port_userapp2 = os.environ["USER_APP2_PORT"]
        self.port_userapp3 = os.environ["USER_APP3_PORT"]
        self.port_userapp4 = os.environ["USER_APP4_PORT"]
        self.port_userapp5 = os.environ["USER_APP5_PORT"]
        
        # wordslab-notebooks install directories
        self.dir_home = os.environ["WORDSLAB_HOME"]
        self.dir_scripts = os.environ["WORDSLAB_SCRIPTS"]     
        self.dir_python = os.environ["UV_PYTHON_INSTALL_DIR"]  
        self.dir_workspace = os.environ["WORDSLAB_WORKSPACE"]
        self.dir_models = os.environ["WORDSLAB_MODELS"]

        # wordslab-notebooks applications install directories
        self.dir_openwebui = os.environ["OPENWEBUI_ENV"]
        self.dir_jupyterlab = os.environ["JUPYTERLAB_ENV"]
        self.dir_vscode = os.environ["VSCODE_DIR"]
        self.dir_ollama = os.environ["OLLAMA_DIR"]
        self.dir_docling = os.environ["DOCLING_ENV"]  

        # wordslab-notebooks applications data directories
        self.dir_openwebui_data = os.environ["OPENWEBUI_DATA"]
        self.dir_jupyterlab_data = os.environ["JUPYTER_DATA"]
        self.dir_vscode_data = os.environ["VSCODE_DATA"]
        self.dir_docling_data = os.environ["DOCLING_DATA"]
        
        # wordslab-notebooks models directories
        self.dir_models_ollama = os.environ["OLLAMA_MODELS"] 
        self.dir_models_vllm = os.environ["HF_HOME"]
        self.dir_models_hugginface = os.environ["HF_HOME"]
        self.dir_models_fastai = os.environ["FASTAI_HOME"] 
        self.dir_models_pytorch = os.environ["TORCH_HOME"] 
        self.dir_models_keras = os.environ["KERAS_HOME"]
        self.dir_models_tensorflow = os.environ["TFHUB_CACHE_DIR"]         
        self.dir_models_docling = os.environ["DOCLING_MODELS"]
        
        # wordslab-notebooks default models
        self.default_model_chat = os.environ["OLLAMA_CHAT_MODEL"]
        self.default_model_embedding = os.environ["OLLAMA_EMBED_MODEL"]
        self.default_model_code = os.environ["OLLAMA_CODE_MODEL"]
        self.default_model_autocomplete = os.environ["OLLAMA_COMPLETION_MODEL"]
